/**
  ******************************************************************************
  * @file     TIMER_CFG.c
  * @author   sinomcu.com-AE
  * @brief    Timer config
  *    
  *          This file provides functions to TIM1 TIM3 CH1 PWM out config:
  *              PA8      --> TIM1_CH1
  *              PA11     --> TIM1_CH4
  *              PA6      --> TIM3_CH1
  *             PWM period 1KHz,Duty 25%. 
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */
    
#include "TIMER_CFG.h"
#include "user_config.h"

/**
  * @brief  Timer1 CH1 PWM Out Pin Initialization
  * @param  None
  * @retval None
  */
void timer1_pwm_pin_init(void) {
  MS32_GPIO_InitTypeDef GpioInitStruct = {0};
  // PA8      --> TIM1_CH1
  // PA11     --> TIM1_CH4
  MS32_GPIO_StructInit(&GpioInitStruct); 
  GpioInitStruct.Pin = MS32_GPIO_PIN_8 |MS32_GPIO_PIN_11;
  GpioInitStruct.Mode = MS32_GPIO_MODE_ALTERNATE;
  GpioInitStruct.Speed = MS32_GPIO_SPEED_FREQ_HIGH;
  GpioInitStruct.OutputType = MS32_GPIO_OUTPUT_PUSHPULL;
  GpioInitStruct.Pull = MS32_GPIO_PULL_UP;
  GpioInitStruct.Alternate = MS32_GPIO_AF_2;
  MS32_GPIO_Init(GPIOA, &GpioInitStruct);                  // init function had enable clock
}

/**
  * @brief  Timer1 CH1 CH1N PWM Initialization
  * @param  None
  * @retval None
  */
void timer1_pwm_init(void) {
  MS32_TIM_InitTypeDef TimerInitStruct = {0};
  MS32_TIM_OC_InitTypeDef TimerOCInitStruct = {0};

  timer1_pwm_pin_init();  
  MS32_TIM_SetClockSource(TIM1, MS32_TIM_CLOCKSOURCE_INTERNAL);                                    // CNT clk form RCC

  // count up,PWM period 1KHz
  MS32_TIM_StructInit(&TimerInitStruct);
  TimerInitStruct.CounterMode = MS32_TIM_COUNTERMODE_UP;
  TimerInitStruct.Prescaler = __MS32_TIM_CALC_PSC(SystemCoreClock,1000000);                        // unit Hz,div sysclk to  CNT clk:1Mhz
  TimerInitStruct.Autoreload = __MS32_TIM_CALC_ARR(SystemCoreClock,TimerInitStruct.Prescaler,1000);// unit Hz,1KHz
  TimerInitStruct.RepetitionCounter = 0;
  MS32_TIM_Init(TIM1, &TimerInitStruct);                   // init function had enable clock
  MS32_TIM_EnableARRPreload(TIM1);

  MS32_TIM_OC_StructInit(&TimerOCInitStruct);
  TimerOCInitStruct.OCMode = MS32_TIM_OCMODE_PWM1;                                                 // PWM mode
  TimerOCInitStruct.OCState = MS32_TIM_OCSTATE_DISABLE;
  TimerOCInitStruct.CompareValue =  ((( MS32_TIM_GetAutoReload(TIM1) + 1 ) * 25 ) / 100);          // set duty cycle 25%
  TimerOCInitStruct.OCPolarity = MS32_TIM_OCPOLARITY_HIGH;
  TimerOCInitStruct.OCIdleState = MS32_TIM_OCIDLESTATE_LOW;
  MS32_TIM_OC_Init(TIM1, MS32_TIM_CHANNEL_CH1, &TimerOCInitStruct);
  MS32_TIM_OC_EnablePreload(TIM1, MS32_TIM_CHANNEL_CH1);
  MS32_TIM_OC_DisableFast(TIM1, MS32_TIM_CHANNEL_CH1);

#if (TIMER_SYNC_MODE == TIM1_TIM3_INDEPENDENT) 
  MS32_TIM_SetTriggerOutput(TIM1, MS32_TIM_TRGO_RESET);
  MS32_TIM_DisableMasterSlaveMode(TIM1);
#elif (TIMER_SYNC_MODE == TIM1_UPDATE_TO_TIM3) 
  MS32_TIM_SetTriggerOutput(TIM1,MS32_TIM_TRGO_UPDATE);
  MS32_TIM_EnableMasterSlaveMode(TIM1);
#else
  #if(TIMER_SYNC_MODE == TIM1_OC4REF_PWM1_TO_TIM3) 
  TimerOCInitStruct.OCMode = MS32_TIM_OCMODE_PWM1;
  #elif(TIMER_SYNC_MODE == TIM1_OC4REF_PWM2_TO_TIM3)   
  TimerOCInitStruct.OCMode = MS32_TIM_OCMODE_PWM2;   
  #endif    
  TimerOCInitStruct.CompareValue =  ((( MS32_TIM_GetAutoReload(TIM1) + 1 ) * 25 ) / 100);          // set duty cycle 25%
  MS32_TIM_OC_Init(TIM1, MS32_TIM_CHANNEL_CH4, &TimerOCInitStruct);
  MS32_TIM_SetTriggerOutput(TIM1, MS32_TIM_TRGO_OC4REF);
  MS32_TIM_EnableMasterSlaveMode(TIM1);
  MS32_TIM_CC_EnableChannel(TIM1, MS32_TIM_CHANNEL_CH4);                                           // enable channel 
#endif
  MS32_TIM_CC_EnableChannel(TIM1, MS32_TIM_CHANNEL_CH1);                                           // enable channel 
  MS32_TIM_EnableAllOutputs(TIM1);                                                                 // enable ouput 
} 
/**
  * @brief  Timer3 CH1 PWM Out Pin Initialization
  * @param  None
  * @retval None
  */
void timer3_pwm_pin_init(void) {
  MS32_GPIO_InitTypeDef GpioInitStruct = {0};
  // PA6      --> TIM3_CH1
  MS32_GPIO_StructInit(&GpioInitStruct); 
  GpioInitStruct.Pin = MS32_GPIO_PIN_6;
  GpioInitStruct.Mode = MS32_GPIO_MODE_ALTERNATE;
  GpioInitStruct.Speed = MS32_GPIO_SPEED_FREQ_HIGH;
  GpioInitStruct.OutputType = MS32_GPIO_OUTPUT_PUSHPULL;
  GpioInitStruct.Pull = MS32_GPIO_PULL_UP;
  GpioInitStruct.Alternate = MS32_GPIO_AF_1;
  MS32_GPIO_Init(GPIOA, &GpioInitStruct);                  // init function had enable clock
}

/**
  * @brief  Timer3 CH1 PWM Initialization
  * @param  None
  * @retval None
  */
void timer3_pwm_init(void) {
  MS32_TIM_InitTypeDef TimerInitStruct = {0};
  MS32_TIM_OC_InitTypeDef TimerOCInitStruct = {0};

  timer3_pwm_pin_init();                                                                           // PWM pin set

  MS32_TIM_SetClockSource(TIM3, MS32_TIM_CLOCKSOURCE_INTERNAL);                                    // CNT clk form RCC

  // count up,PWM period 1KHz
  MS32_TIM_StructInit(&TimerInitStruct);
  TimerInitStruct.CounterMode = MS32_TIM_COUNTERMODE_UP;
  TimerInitStruct.Prescaler = __MS32_TIM_CALC_PSC(SystemCoreClock,1000000);                        // unit Hz,div sysclk to  CNT clk:1Mhz
  TimerInitStruct.Autoreload = __MS32_TIM_CALC_ARR(SystemCoreClock,TimerInitStruct.Prescaler,1000);// unit Hz,1KHz
  MS32_TIM_Init(TIM3, &TimerInitStruct);                   // init function had enable clock
  MS32_TIM_EnableARRPreload(TIM3);

  MS32_TIM_OC_StructInit(&TimerOCInitStruct);
  TimerOCInitStruct.OCMode = MS32_TIM_OCMODE_PWM1;                                                 // PWM1 mode
  TimerOCInitStruct.OCState = MS32_TIM_OCSTATE_DISABLE;
  TimerOCInitStruct.CompareValue =  ((( MS32_TIM_GetAutoReload(TIM3) + 1 ) * 25 ) / 100);          // set duty cycle 25%
  TimerOCInitStruct.OCPolarity = MS32_TIM_OCPOLARITY_HIGH;
  MS32_TIM_OC_Init(TIM3, MS32_TIM_CHANNEL_CH1, &TimerOCInitStruct);
  MS32_TIM_OC_EnablePreload(TIM3, MS32_TIM_CHANNEL_CH1);
  MS32_TIM_OC_DisableFast(TIM3, MS32_TIM_CHANNEL_CH1);

#if (TIMER_SYNC_MODE == TIM1_TIM3_INDEPENDENT) 
  MS32_TIM_SetTriggerOutput(TIM3, MS32_TIM_TRGO_RESET);
  MS32_TIM_DisableMasterSlaveMode(TIM3);
#else
  MS32_TIM_SetSlaveMode(TIM3,MS32_TIM_SLAVEMODE_TRIGGER);
  MS32_TIM_SetTriggerInput(TIM3, MS32_TIM_TS_ITR0);
#endif

  MS32_TIM_CC_EnableChannel(TIM3, MS32_TIM_CHANNEL_CH1);                                           // enable channel 
  MS32_TIM_EnableAllOutputs(TIM3);                                                                 // enable ouput 
  // MS32_TIM_GenerateEvent_UPDATE(TIM3);                                                          // Force update
} 

/******************************** END OF FILE *********************************/
